#------------------------------------------------------------------------
# -*- mode: cmake -*-
#
#------------------------------------------------------------------------
function(ax_add_library target)
  #----------------------------------------------------------------------
  # Handle argument parsing
  #----------------------------------------------------------------------
  set( options )
  set( oneValueArgs
       C_STANDARD_REQUIRED
       SOVERSION
  )
  set( multiValueArgs
       DEFINES
       DEPENDENCIES
       LINK_LIBRARIES
       SOURCES
  )
  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN} )
  string( TOUPPER "${target}" target_uc )
  #----------------------------------------------------------------------
  # Establish where to install shared libraries
  #----------------------------------------------------------------------
  if ( WIN32 )
    set( RUNTIME_LIB_DIR ${INSTALL_BIN_DIR} )
  else ( WIN32 )
    set( RUNTIME_LIB_DIR ${INSTALL_LIB_DIR} )
  endif ( WIN32 )
  #----------------------------------------------------------------------
  # Generate shared and/or static libraries
  #----------------------------------------------------------------------
  add_library( ${target} SHARED ${ARG_SOURCES} )
  add_library( ${target}Static STATIC ${ARG_SOURCES} )
  if ( WIN32 )
    set_target_properties( ${target}Static PROPERTIES OUTPUT_NAME ${target}S )
  else( WIN32 )
    set_target_properties( ${target}Static PROPERTIES OUTPUT_NAME ${target} )
  endif ( WIN32 )
  if(ARG_SOVERSION)
    if ( UNIX )
      set_target_properties( ${target} PROPERTIES SOVERSION ${ARG_SOVERSION})
    endif ( UNIX )
  endif(ARG_SOVERSION)
  if ( ARG_C_STANDARD_REQUIRED )
    #--------------------------------------------------------------------
    # See if this can be handled via property
    #--------------------------------------------------------------------
    get_property( STANDARD_PROP VARIABLE
  		  PROPERTY C_STANDARD_REQUIRED DEFINED
    )
    if ( STANDARD_PROP )
      set_target_properties( ${target} ${target}Static
      			     PROPERTIES
      			       C_STANDARD_REQUIRED ${ARG_C_STANDARD_REQUIRED}
    )
    else ( STANDARD_PROP )
      set_target_properties( ${target} ${target}Static
      			     PROPERTIES
      			       COMPILE_FLAGS "--std=c${ARG_C_STANDARD_REQUIRED}"
    )
    endif ( STANDARD_PROP )
  endif ( ARG_C_STANDARD_REQUIRED )
  if ( ARG_DEFINES )
    target_compile_definitions( ${target} PRIVATE ${ARG_DEFINES} )
    target_compile_definitions( ${target}Static PRIVATE ${ARG_DEFINES} )
  endif ( ARG_DEFINES )
  if( ARG_DEPENDENCIES )
    add_dependencies( ${target} ${ARG_DEPENDENCIES} )
    add_dependencies( ${target}Static ${ARG_DEPENDENCIES} )
  endif( ARG_DEPENDENCIES )
  if ( ARG_LINK_LIBRARIES )
    target_link_libraries( ${target} ${ARG_LINK_LIBRARIES} )
  endif ( ARG_LINK_LIBRARIES )  
  set( ${target_uc}_PC_LIBS
       #"${CMAKE_INSTALL_PREFIX}/lib/${CMAKE_SHARED_LIBRARY_PREFIX}${target}${CMAKE_SHARED_LIBRARY_SUFFIX}"
       "-L\${libdir} -l${target}"
  )
  set( ${target_uc}_PC_LIBS
       "${${target_uc}_PC_LIBS}"
       CACHE INTERNAL ""
  )
  message( STATUS "lib_PC_LIBS: ${target_uc}: ${${target_uc}_PC_LIBS}" )
  #----------------------------------------------------------------------
  # Configure for pkg-config
  #----------------------------------------------------------------------
  #if ( TRUE )
    configure_file( ${CMAKE_SOURCE_DIR}/cmake/virgo${target}.pc.in
                    ${CMAKE_CURRENT_BINARY_DIR}/virgo${target}.pc
		    @ONLY
   )
   configure_file( ${CMAKE_SOURCE_DIR}/cmake/virgo${target}-uninstalled.pc.in
                ${CMAKE_CURRENT_BINARY_DIR}/virgo${target}-uninstalled.pc
		@ONLY
   )
   install(FILES ${CMAKE_CURRENT_BINARY_DIR}/virgo${target}.pc
	DESTINATION "${INSTALL_LIB_DIR}/pkgconfig"
	COMPONENT Development
  )
  #endif (TRUE )
  #----------------------------------------------------------------------
  # Install the libraries
  #----------------------------------------------------------------------
  install( TARGETS
	     ${target} ${target}Static
           RUNTIME
             DESTINATION ${RUNTIME_LIB_DIR}
             COMPONENT Applications
          LIBRARY
            DESTINATION ${INSTALL_LIB_DIR}
            COMPONENT Applications
          ARCHIVE
            DESTINATION ${INSTALL_LIB_DIR}
            COMPONENT Development
  )
endfunction(ax_add_library)
