#------------------------------------------------------------------------
# -*- mode: cmake -*-
#
#------------------------------------------------------------------------
function(ax_application target)
  #----------------------------------------------------------------------
  # Handle argument parsing
  #----------------------------------------------------------------------
  set( options
       NO_INSTALL
       INSTALL_EXE
  )
  set( oneValueArgs
       C_STANDARD_REQUIRED
       DESTINATION
       OUTPUT_NAME
  )
  set( multiValueArgs
       DEFINES
       LINK_LIBRARIES
       SOURCES
  )
  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN} )
  string( TOUPPER "${target}" target_uc )
  if ( NOT ARG_DESTINATION )
    set( ARG_DESTINATION ${INSTALL_BIN_DIR} )
  endif ( NOT ARG_DESTINATION )
  #----------------------------------------------------------------------
  # Generate executable
  #----------------------------------------------------------------------
  add_executable( ${target} ${ARG_SOURCES} )
  if ( ARG_C_STANDARD_REQUIRED )
    #--------------------------------------------------------------------
    # See if this can be handled via property
    #--------------------------------------------------------------------
    get_property( STANDARD_PROP VARIABLE
  		  PROPERTY C_STANDARD_REQUIRED DEFINED
    )
    if ( STANDARD_PROP )
      set_target_properties( ${target}
      			     PROPERTIES
      			       C_STANDARD_REQUIRED ${ARG_C_STANDARD_REQUIRED}
    )
    else ( STANDARD_PROP )
      #------------------------------------------------------------------
      # Currently only works for GNU like compilers
      #------------------------------------------------------------------
      set_target_properties( ${target}
      			     PROPERTIES
      			       COMPILE_FLAGS "--std=c${ARG_C_STANDARD_REQUIRED}"
    )
    endif ( STANDARD_PROP )
  endif ( ARG_C_STANDARD_REQUIRED )
  if ( ARG_DEFINES )
    target_compile_definitions( ${target} PRIVATE ${ARG_DEFINES} )
  endif ( ARG_DEFINES )
  if ( ARG_LINK_LIBRARIES )
    target_link_libraries( ${target} ${ARG_LINK_LIBRARIES} )
  endif ( ARG_LINK_LIBRARIES )
  if ( ARG_OUTPUT_NAME )
    set_target_properties( ${target}
    			   PROPERTIES
    			     OUTPUT_NAME ${ARG_OUTPUT_NAME}
    )
  endif ( ARG_OUTPUT_NAME )
  #----------------------------------------------------------------------
  # Install the executable
  #----------------------------------------------------------------------
  if ( NOT ARG_NO_INSTALL )
    get_target_property( INSTALL_TARGET ${target} OUTPUT_NAME )
    if ( INSTALL_TARGET )
      set( INSTALL_TARGET "${CMAKE_CURRENT_BINARY_DIR}/${INSTALL_TARGET}" )
    else( INSTALL_TARGET )
      set( INSTALL_TARGET "${CMAKE_CURRENT_BINARY_DIR}/${target}" )
    endif ( INSTALL_TARGET )
    install( PROGRAMS ${INSTALL_TARGET}
             DESTINATION ${ARG_DESTINATION}
             COMPONENT Applications
    )
  endif ( NOT ARG_NO_INSTALL )
  if ( ARG_INSTALL_EXE AND UNIX )
    message( STATUS "-- Will install exe for ${target}" )
    get_target_property( otarget ${target} OUTPUT_NAME )
    if ( NOT otarget )
      set( otarget ${target} )
    endif ( NOT otarget )
    install( CODE "
    	     	  execute_process( COMMAND ${CMAKE_COMMAND} -E create_symlink ${otarget} ${otarget}.exe
   	    	 		    WORKING_DIRECTORY ${CMAKE_INSTALL_PREFIX}/${INSTALL_BIN_DIR}
                  )
		  message( STATUS \"-- Linked ${INSTALL_BIN_DIR}/${otarget} to ${otarget}.exe\" )
		  "
    )
  endif ( ARG_INSTALL_EXE AND UNIX )
endfunction(ax_application)
